/* =============================================================
 * Interface to DS1340 Real-Time-Clock chip, an I2C bus RTC
 * module.
 * =============================================================
 * By: Zakie Mashiah
 * =============================================================
 */
#include <Arduino.h>
#include <Wire.h>

#include "DS1340.h"


DS1340_I2CRTC::DS1340_I2CRTC()
{
	I2C_address = 0x68;  // I2C write address 
	
	memset(&dt, 0, sizeof(dt));
}


int DS1340_I2CRTC::_fromBCD(byte bcd)
{
  int lsb = bcd & 0x0F;
  int msb = (bcd & 0xF0) >> 4;
  
  return msb * 10 + lsb;
}


byte DS1340_I2CRTC::_toBCD(int value)
{
	return (byte)(((value / 10) << 4) + value % 10);
}


void DS1340_I2CRTC::init() 
{ 
} 
 

/*******************************************************************************
                   Set time function
*******************************************************************************/ 
void DS1340_I2CRTC::setTime(DateAndTime *pdt)
{
	if (pdt != NULL)
		memcpy(&dt, pdt, sizeof(dt) );
}



void DS1340_I2CRTC::setTime(int hour, int minute, int second, int year, int month, int date)
{
	dt.Hour = hour;
	dt.Minute = minute;
	dt.Second = second;
	
	dt.Year = year;
	dt.Month = month;
	dt.Date = date;
}	

void DS1340_I2CRTC::setRTCTime(DateAndTime *pdt)
{
	DateAndTime *ptr;
	
	if (pdt == NULL)
		ptr = &dt;
	else
		ptr = pdt;
	
	Wire.beginTransmission(I2C_address);
	Wire.write((uint8_t)0);
	Wire.write(_toBCD(ptr->Second));		// Convert to BCD when writing to chip
	Wire.write(_toBCD(ptr->Minute));
	Wire.write(_toBCD(ptr->Hour));
	Wire.write(_toBCD(ptr->Day));
	Wire.write(_toBCD(ptr->Date));
	Wire.write(_toBCD(ptr->Month));
	Wire.write(_toBCD(ptr->Year));
	Wire.endTransmission();
}


DateAndTime *DS1340_I2CRTC::getTime(DateAndTime *pdt)
{
	DateAndTime local;
	char tempchar [7];
	byte i = 0;
	
	Wire.beginTransmission(I2C_address);
	Wire.write( (uint8_t)0);
	Wire.endTransmission();
	Wire.requestFrom(I2C_address, 7);
	while(Wire.available())          // Checkf for data from slave
	{ 
		tempchar [i] = Wire.read(); // receive a byte as character 
		i++;
	}
	// Convert the BCD data read from chip to integer format
	local.Second = _fromBCD(tempchar [0]);
	local.Minute = _fromBCD(tempchar [1]);
	local.Hour   = _fromBCD(tempchar [2]);
	local.Day    = _fromBCD(tempchar [3]);
	local.Date   = _fromBCD(tempchar [4]);
	local.Month  = _fromBCD(tempchar [5]);
	local.Year   = _fromBCD(tempchar [6]);
	// and copy to right destination
	if (pdt == NULL)
	{
		memcpy(&dt, &local, sizeof(dt));
		return &dt;
	}
	memcpy(pdt, &local, sizeof(DateAndTime) );
	return pdt;
}

DS1340_I2CRTC 		rtcModule;	// Define one object to represent the RTC chip


